@@ -3,6 +3,7 @@ import multiprocessing
 import os
 import pickle
 from functools import partial, wraps
+from pprint import PrettyPrinter
 
 from pydub.utils import mediainfo
 
@@ -486,7 +487,7 @@ class Audalign:
         Recognizes given file against already fingerprinted files
 
         Offset describes duration that the recognized file aligns after the target file
-        Does not recognize against files with same name and extention
+        Does not recognize against files with same name and extension
         Locality option used to only return match results within certain second range
 
         Args
@@ -1217,7 +1218,10 @@ class Audalign:
 
         if match_index != 0:
             recalc_shifts_results = align.recalc_shifts_index(
-                results, strength_stat=strength_stat, match_index=match_index
+                results,
+                key="match_info",
+                match_index=match_index,
+                strength_stat=strength_stat,
             )
             paths_audio = filehandler.shift_get_files(
                 recalc_shifts_results, sample_rate=cor_sample_rate
@@ -1290,6 +1294,174 @@ class Audalign:
         """
         return datalign.rank_alignment(alignment=alignment)
 
+    @staticmethod
+    def pretty_print_results(results, match_keys="both"):
+        """
+        nifty printer for recognition or alignment results
+        match_keys goes to pretty_print_alignment
+
+        Args
+        ----
+            results (dict): result from recognition or alignment
+            match_keys (str): "both", "match_info", or "fine_match_info"
+
+        Returns
+        -------
+            None
+        """
+        if results:
+            if "names_and_paths" not in results:
+                Audalign.pretty_print_recognition(results)
+            else:
+                Audalign.pretty_print_alignment(results, match_keys=match_keys)
+        else:
+            print("No Matches Found")
+
+    @staticmethod
+    def pretty_print_recognition(results, _in_alignment: bool = False):
+        """
+        nifty printer for recognition results
+
+        Args
+        ----
+            results (dict): result from recognition
+
+        Returns
+        -------
+            None
+        """
+        if results:
+            strength_stat = "confidence"
+            _ = results["match_info"]
+            _ = _[list(_.keys())[0]]
+            if "confidence" not in _:
+                strength_stat = "ssim"
+            elif "scaling_factor" in _:
+                strength_stat = "scaling_factor"
+            audio_file_list = sorted(
+                list(results["match_info"].keys()),
+                key=lambda x: _recog_sorter(x, results, strength_stat),
+            )
+            for audio_file in audio_file_list:
+                if not _in_alignment:
+                    print(f"\n{audio_file}")
+                else:
+                    print(f"\nmatched with {audio_file}")
+                print(f"Match time: {seconds_to_min_hrs(results.get('match_time'))}")
+                for section, info in results["match_info"][audio_file].items():
+                    if type(info) == type([]):
+                        if section in [
+                            "locality_frames",
+                            "offset_frames",
+                            "locality_samples",
+                            "offset_samples",
+                        ]:
+                            continue
+                        elif type(info) == type([]):
+                            if section == "mse":
+                                if info[0] == 20000000:
+                                    continue
+                            elif section == "locality_seconds":
+                                print(f"{section}: [", end="")
+                                for num, i in enumerate(info):
+                                    if num > 9:
+                                        break
+                                    if i is not None:
+                                        print(f"{i[0]} {len(i)} : ", end="")
+                                    else:
+                                        print("None : ", end="")
+                                print("\b\b\b] ")
+                                continue
+                            if info is None or info[0] is None:
+                                continue
+                            print(f"{section}: {info[0:10] if len(info) > 9 else info}")
+                            print(f'max "{section}" is {max(info)}: min is {min(info)}')
+                    else:
+                        print(f"{section}, {info}")
+            if not _in_alignment:
+                print()
+                PrettyPrinter().pprint(results["rankings"])
+        else:
+            print("No Matches Found")
+
+    @staticmethod
+    def pretty_print_alignment(results, match_keys="both"):
+        """
+        nifty printer for alignment results
+        if match_keys is both, prints "match_info" and "fine_match_info" if present
+
+        Args
+        ----
+            results (dict): result from alignment
+            match_keys (str): "both", "match_info", or "fine_match_info"
+
+        Returns
+        -------
+            None
+        """
+        if results:
+            if match_keys == "both":
+                match_keys = ["match_info"]
+                if results.get("fine_match_info") is not None:
+                    match_keys += ["fine_match_info"]
+            else:
+                match_keys = [match_keys]
+            for match_key in match_keys:
+                print(f'\n        Key: "{match_key}"')
+                for audio_file in results.get(match_key).keys():
+                    print(f"\n    {audio_file}")
+                    Audalign.pretty_print_recognition(
+                        results[match_key][audio_file], _in_alignment=True
+                    )
+            print("\nRankings")
+            PrettyPrinter().pprint(results["rankings"])
+            print()
+            for match in results.keys():
+                if match not in [
+                    "match_info",
+                    "fine_match_info",
+                    "names_and_paths",
+                    "rankings",
+                ]:
+                    print(f"{match} : {results[match]}")
+        else:
+            print("No Matches Found")
+        print()
+
+    @staticmethod
+    def recalc_shifts(
+        results: dict,
+        key: str = None,
+        match_index: int = 0,
+        fine_match_index: int = 0,
+        strength_stat: str = None,
+        fine_strength_stat: str = None,
+    ):
+        """
+        Takes results from alignment and recalculates alignment
+
+        key is either "match_info", "fine_match_info", or "only_fine_match_info"
+        "match_info" is like undoing the fine alignment. "fine_match_info" is like redoing the fine alignment.
+        "only_fine_match_info" gives the relative shifts only calculated by fine alignment without the first alignment.
+
+        Args
+        ----
+            results (dict): results from alignment.
+            key (int, optional): key to recalculate from. defaults to "fine_match_info" if present.
+            match_index (int, optional): index of match to reorder by. First index (0) is usually the strongest match.
+            fine_match_index (int, optional): index of match to reorder by for fine_match_info. First index (0) is usually the strongest match.
+            strength_stat (str, optional): change if using mse for visual *not recommended*.
+            fine_strength_stat (str, optional): change if using mse for visual *not recommended*.
+        """
+        return align.recalc_shifts_index(
+            results,
+            key=key,
+            match_index=match_index,
+            fine_match_index=fine_match_index,
+            strength_stat=strength_stat,
+            fine_strength_stat=fine_strength_stat,
+        )
+
     @staticmethod
     def _write_shifted_files(
         files_shifts: dict,
@@ -1341,6 +1513,68 @@ class Audalign:
         """
         filehandler.shift_write_file(file_path, destination_path, offset_seconds)
 
+    @staticmethod
+    def write_shifts_from_results(
+        results: dict, read_from_dir, destination_path: str, write_extension: str = None
+    ):
+        """
+        For writing the results of an alignment with alternate source files.
+        Especially useful if you want to align the original files with the results from noise
+        removed or uniform leveled versions.
+
+        Read_from_dir can be a str of directory, a list of file path str's, or None to use original source files.
+
+        Cannot have two files with same basename ("Test.wav" and "Test.mp3" in same read_from_dir is undefined)
+
+        Args
+        ----
+            results (dict): results from alignment.
+            read_from_dir (str, list): source files for writing. or None to use original files.
+            destination_path (str): destination to write to.
+            write_extension (str, optional): if given, all files writen with given extension
+        """
+        if isinstance(read_from_dir, str):
+            print("Finding audio files")
+            read_from_dir = filehandler.get_audio_files_directory(
+                read_from_dir, full_path=True
+            )
+        if read_from_dir is not None:
+            results_files = {}
+            for path in results.keys():
+                if len(os.path.splitext(os.path.basename(path))[1]) > 0:
+                    results_files[os.path.splitext(os.path.basename(path))[0]] = path
+
+            copy_dict = {}
+            names_and_paths = {}
+            for filename in read_from_dir:
+                base_basename = os.path.splitext(os.path.basename(filename))[0]
+                if base_basename in results_files:
+                    copy_dict[base_basename] = results[results_files[base_basename]]
+                    names_and_paths[base_basename] = filename
+            if copy_dict == {}:
+                print("No matching file basenames found")
+                return
+        else:
+            copy_dict = {}
+            for name, value in results.items():
+                if name not in [
+                    "names_and_paths",
+                    "match_info",
+                    "fine_match_info",
+                    "rankings",
+                ]:
+                    copy_dict[name] = value
+            names_and_paths = results["names_and_paths"]
+        try:
+            Audalign._write_shifted_files(
+                copy_dict,
+                destination_path,
+                names_and_paths,
+                write_extension,
+            )
+        except PermissionError:
+            print("Permission Denied for write fine_align")
+
     @staticmethod
     def convert_audio_file(
         file_path: str,
@@ -1382,7 +1616,7 @@ class Audalign:
         ----
             file_path (str): path of file to level.
             destination (str): path of destination to write to. directory or file name.
-            write_extension (str, optional): extention/format for writing.
+            write_extension (str, optional): extension/format for writing.
             mode (str): either "normalize" or "average".
             width (float): width in seconds for each leveling.
             overlap_ratio (float): between 1 and 0. overlapping windows.
@@ -1424,7 +1658,7 @@ class Audalign:
         ----
             file_path (str): path of file to level.
             destination (str): path of destination to write to. directory or file name.
-            write_extension (str, optional): extention/format for writing.
+            write_extension (str, optional): extension/format for writing.
             mode (str): either "normalize" or "average".
             width (float): width in seconds for each leveling.
             overlap_ratio (float): between 1 and 0. overlapping windows.
@@ -1530,3 +1764,25 @@ class Audalign:
             num_processes=self.num_processors,
             **kwargs,
         )
+
+
+def seconds_to_min_hrs(seconds):
+    if seconds > 60:
+        minutes = seconds // 60
+        seconds = seconds % 60
+        if minutes > 60:
+            hours = minutes // 60
+            minutes = minutes % 60
+            return (
+                f"{int(hours)} hours, {int(minutes)} minutes, and {seconds:.3f} seconds"
+            )
+        else:
+            return f"{int(minutes)} minutes and {seconds:.3f} seconds"
+    else:
+        return f"{seconds:.3f} seconds"
+
+
+def _recog_sorter(x, result, strength_stat):
+    if strength_stat == "scaling_factor":
+        return result["match_info"][x][strength_stat]
+    return result["match_info"][x][strength_stat][0]
