@@ -50,7 +50,6 @@ def _align(
             target_aligning=target_aligning,
             target_start_end=target_start_end,
             fine_aud_file_dict=fine_aud_file_dict,
-            cor_sample_rate=cor_sample_rate,
         )
 
         file_list, dir_or_list = set_list_and_dir(
@@ -91,6 +90,7 @@ def _align(
             file_names_and_paths=file_names_and_paths,
             write_extension=write_extension,
             target_aligning=target_aligning,
+            fine_aligning=fine_aud_file_dict is not None,
         )
 
         if not files_shifts:
@@ -133,7 +133,6 @@ def set_ada_file_names(
     target_aligning,
     target_start_end,
     fine_aud_file_dict,
-    cor_sample_rate,
 ):
     # Make target directory
     if destination_path:
@@ -167,14 +166,6 @@ def set_ada_file_names(
             ada_obj.file_names = [
                 os.path.basename(x) for x in fine_aud_file_dict.keys()
             ]
-            # if technique in ["correlation", "correlation_spectrogram"]:
-            # TODO could remove for multiprocessing in the future
-            # for path in fine_aud_file_dict.keys():
-            #     fine_aud_file_dict[path] = audalign.filehandler.get_shifted_file(
-            #         path,
-            #         fine_aud_file_dict[path],
-            #         sample_rate=cor_sample_rate,
-            #     )
         else:
             ada_obj.file_names = [os.path.basename(x) for x in filename_list]
 
@@ -354,6 +345,7 @@ def calc_final_alignments(
     file_names_and_paths,
     write_extension,
     target_aligning,
+    fine_aligning: bool,
 ):
     if ada_obj.alternate_strength_stat is not None:
         if technique == "fingerprints":
@@ -380,6 +372,14 @@ def calc_final_alignments(
                 ].keys()
             ):
                 file_names_and_paths[os.path.basename(file_path)] = file_path
+    if fine_aligning is False:
+        max_shift = max(files_shifts.values())
+        for name in files_shifts.keys():
+            files_shifts[name] = max_shift - files_shifts[name]
+    else:
+        min_shift = min(files_shifts.values())
+        for name in files_shifts.keys():
+            files_shifts[name] = files_shifts[name] - min_shift
 
     if destination_path:
         try:
@@ -499,8 +499,12 @@ def find_matches_not_in_file_shifts(
 def combine_fine(results: dict, new_results: dict):
     fine_match_info = new_results.pop("match_info")
     temp_names_and_paths = new_results.pop("names_and_paths")
+    max_shift = max(new_results.values())
     for name in new_results.keys():
-        new_results[name] += results[name]
+        new_results[name] = max_shift - new_results[name] + results[name]
+    min_shift = min(new_results.values())
+    for name in new_results.keys():
+        new_results[name] = new_results[name] - min_shift
     new_results["fine_match_info"] = fine_match_info
     new_results["match_info"] = results["match_info"]
     new_results["names_and_paths"] = temp_names_and_paths
@@ -508,19 +512,88 @@ def combine_fine(results: dict, new_results: dict):
 
 
 def recalc_shifts_index(
-    results: dict, match_index: int, strength_stat: str = "confidence"
-):
+    results: dict,
+    key=None,
+    match_index: int = 0,
+    fine_match_index: int = 0,
+    strength_stat: str = None,
+    fine_strength_stat=None,
+) -> dict:
+    if key is None:
+        if "fine_match_info" in results:
+            key = "fine_match_info"
+        else:
+            key = "match_info"
+    if key not in ["match_info", "fine_match_info", "only_fine_match_info"]:
+        raise ValueError(
+            f'key must be "match_info", "fine_match_info", or "only_fine_match_info", not "{key}"'
+        )
+    temp_info_key = "match_info" if key != "only_fine_match_info" else "fine_match_info"
+
+    if strength_stat is None:
+        strength_stat = "confidence"
+        _ = results[temp_info_key]
+        _ = _[list(_.keys())[0]]
+        _ = _["match_info"]
+        _ = _[list(_.keys())[0]]
+        if "confidence" not in _.keys():
+            strength_stat = "ssim"
+
+    files_shifts = _calc_shifts_index(
+        results[temp_info_key], strength_stat, match_index=match_index
+    )
+    if temp_info_key == "match_info":
+        max_shift = max(files_shifts.values())
+        for name in files_shifts.keys():
+            files_shifts[name] = max_shift - files_shifts[name]
+    else:
+        min_shift = min(files_shifts.values())
+        for name in files_shifts.keys():
+            files_shifts[name] = files_shifts[name] - min_shift
+    for name in files_shifts.keys():
+        results[name] = files_shifts[name]
+
+    if key == "fine_match_info":
+        if fine_strength_stat is None:
+            fine_strength_stat = "confidence"
+            _ = results["fine_match_info"]
+            _ = _[list(_.keys())[0]]
+            _ = _["match_info"]
+            _ = _[list(_.keys())[0]]
+            if "confidence" not in _.keys():
+                fine_strength_stat = "ssim"
+        files_shifts = _calc_shifts_index(
+            results["fine_match_info"], fine_strength_stat, match_index=fine_match_index
+        )
+        max_shift = max(files_shifts.values())
+        for name in files_shifts.keys():
+            files_shifts[name] = max_shift - files_shifts[name] + results[name]
+        min_shift = min(files_shifts.values())
+        for name in files_shifts.keys():
+            results[name] = files_shifts[name] - min_shift
+
+    for name in results.keys():
+        if name not in [
+            "match_info",
+            "fine_match_info",
+            "rankings",
+            "names_and_paths",
+            *list(files_shifts.keys()),
+        ]:
+            results.pop(name)
+
+    return results
+
+
+def _calc_shifts_index(info_to_use, strength_stat, match_index):
     files_shifts = find_most_matches(
-        results["match_info"], strength_stat=strength_stat, match_index=match_index
+        info_to_use, strength_stat=strength_stat, match_index=match_index
     )
     if not files_shifts:
         return
-    files_shifts = find_matches_not_in_file_shifts(
-        results["match_info"],
+    return find_matches_not_in_file_shifts(
+        info_to_use,
         files_shifts,
         strength_stat=strength_stat,
         match_index=match_index,
     )
-    for name in files_shifts.keys():
-        results[name] = files_shifts[name]
-    return results
