@@ -59,7 +59,7 @@ def correcognize(
     if max_lags is not None:
         max_lags = int(calc_spec_windows(max_lags, sample_rate, technique))
     if filter_matches is None:
-        filter_matches = 0.5
+        filter_matches = 0.0
     if locality is not None:
         locality = int(calc_spec_windows(locality, sample_rate, technique))
     if locality_filter_prop is None:
@@ -109,7 +109,7 @@ def correcognize(
     t = time.time() - t
 
     result = {}
-    if file_match:
+    if len(file_match) > 0:
         result["match_time"] = t
         result["match_info"] = file_match
         return result
@@ -159,7 +159,7 @@ def correcognize_directory(
     if _file_audsegs is not None and filter_matches is None:
         filter_matches = 0.0
     elif filter_matches is None:
-        filter_matches = 0.5
+        filter_matches = 0.0
 
     sos = signal.butter(
         10, fingerprint.threshold, "highpass", fs=sample_rate, output="sos"
@@ -222,7 +222,7 @@ def correcognize_directory(
     t = time.time() - t
 
     result = {}
-    if file_match:
+    if len(file_match) > 0:
         result["match_time"] = t
         result["match_info"] = file_match
         if _include_filename:
@@ -411,7 +411,9 @@ def calc_array_indexes(array, locality):
     else:
         [
             index_list.append(i)
-            for i in range(0, len(array) - int(locality), int(locality * OVERLAP_RATIO))
+            for i in range(
+                0, len(array) - int(locality), int(locality * (1 - OVERLAP_RATIO))
+            )
         ]
         if (
             len(array) - int(locality) not in index_list
@@ -745,7 +747,7 @@ def process_results(
 
     if len(match[file_name]["offset_seconds"]) > 0:
         return match
-    return None
+    return {}
 
 
 # ---------------------------------------------------------------------------------
